@@ -1,5 +1,4 @@
 from typing import List, Dict, Tuple, Union
-import os
 import random
 import re
 import uuid
@@ -548,7 +547,6 @@ input {
 
 #log-tags {
     display: grid;
-    grid-template-columns: repeat(4, auto);
     grid-gap: 18px;
     justify-content: space-evenly;
     align-items: center;
@@ -882,6 +880,10 @@ input:checked + .slider:before {
 """
 
 class HtmlLogGenerator:
+    """
+    Generates an HTML log file from the given logs. The logs must derive from the logger in the
+    format returned by ``Logger.get_logs()``.
+    """
     class _columns:
         file_path = 'file_path'
         function_name = 'function_name'
@@ -900,28 +902,27 @@ class HtmlLogGenerator:
         self._log_entries = logs['log_entries']
         self._main_thread_id = logs['main_thread_id']
 
-    def generate(self, log_file: Union[str, Path], title: str = 'Log Results', include_code: bool = True,
+    def generate(self, log_file: Union[str, Path], title: str = 'Log Results', include_code: bool = False,
                  datetime_range: Tuple[datetime, datetime] = None, exclude_files: List[str] = None):
         """
-        Generates an HTML file that display logs persisted by the logger in the SQLite DB. Consider the
-        parameter definitions for ways to reduce the size of the output and control the content in the
-        output.
+        Generates an HTML log file from the logs persisted by the logger.
+
+        If using a compiling tool such as PyInstaller, ``include_code`` must be ``False``. Also, there is more
+        overhead when including the code snippets associated to the log events. Each log event is associated to a
+        file; each file is imported in HTML format as an iframe to the overall output.
 
         Args:
             log_file: Absolute path to the SQLite DB log file.
             title: Title of the log file. Default is "Log File".
-            include_code: If ``False``, then the code blocks are not included in the output file. This
-                          is desirable when trying to save space in the output and/or excluding all
-                          files for security purposes. If code blocks are not included, an Info block
-                          is still included in each log entry that displays the file name and line number
-                          of where the log message was recorded.
+            include_code: If ``True`` the include iframes of the source code of the files associated to the log
+                          entries.
             datetime_range: Defines the start and/or end date parameters of the log data included in the
                             output file. This is always a ``Tuple`` of size 2 where the first item is the
                             start date and the second is the end date. If only a start or end date is
                             desired, then set the other value to ``None``. Date should be ``datetime``
                             objects.
             exclude_files: A list of regular expressions for files that should NOT be included in the
-                           code blocks. If ``include_code=True``, then the code files are included in
+                           code blocks. If ``include_code`` is ``True``, then the code files are included in
                            the output so the code can be referenced directly by the logs. This parameter
                            can be used to improve security and space. It is recommended to exclude files
                            that may leak sensitive information and are highly unlikely to provide much
@@ -956,9 +957,9 @@ class HtmlLogGenerator:
             thread_starts = [(t, le[0][self._columns.timestamp]) for t, le in temp_log_entries.items()
                              if not le[0][self._columns.is_main_thread]]
             thread_starts = sorted(thread_starts, key=lambda x: x[1])
-            thread_id, timestamp = thread_starts.pop()
 
             if self._main_thread_id in temp_log_entries.keys():
+                thread_id, timestamp = thread_starts.pop()
                 for entry in temp_log_entries.pop(self._main_thread_id):
                     while timestamp and entry[self._columns.timestamp] >= timestamp:
                         log_entries += temp_log_entries.pop(thread_id)
@@ -975,6 +976,15 @@ class HtmlLogGenerator:
             log_entries = temp_log_entries[self._main_thread_id]
         code_blocks = self._get_code_blocks(log_entries=log_entries,
                                             exclude_files=exclude_files) if include_code else None
+
+
+        used_log_tags, log_entries_html = self._get_log_entries(log_entries=log_entries,
+                              code_blocks=code_blocks if include_code else None)
+        self._log_tags = {k: v for k, v in self._log_tags.items() if k in used_log_tags}
+        code_blocks_html = ''.join([cb['block'] for cb in code_blocks.values()]) if code_blocks else ''
+        legend_html = self._legend()
+        additional_styles_css = self._get_styles(include_code=include_code)
+
         # Minify HTML file to reduce space.
         html = htmlmin.minify(f"""
         <html>
@@ -985,7 +995,7 @@ class HtmlLogGenerator:
                 </script>
                 <style>
                     {STYLES_CSS}
-                    {self._get_styles(include_code=include_code)}
+                    {additional_styles_css}
                 </style>
             </head>
             <body>
@@ -995,16 +1005,15 @@ class HtmlLogGenerator:
                 </div>
                 <!-- Legend -->
                 <div id="legend">
-                    {self._legend()}
+                    {legend_html}
                 </div>
                 <!-- Code Blocks -->
                 <div class="hide" id="code-blocks">
-                    {''.join([cb['block'] for cb in code_blocks.values()]) if code_blocks else ''}
+                    {code_blocks_html}
                 </div>
                 <!-- Log Entries -->
                 <div id="log-entries">
-                    {self._get_log_entries(log_entries=log_entries,
-                                           code_blocks=code_blocks if include_code else None)}
+                    {log_entries_html}
                 </div>
             </body>
             {'<br/>' * 12}
@@ -1013,20 +1022,15 @@ class HtmlLogGenerator:
         # endregion Create HTML File
 
         # region Send Files To Log Directory
-        resources = f'{log_file.parent}'
-        file_path = ''
-        for part in resources.split(os.sep):
-            file_path += part + os.sep
-            if not os.path.exists(file_path):
-                os.mkdir(file_path)
-
+        resources = log_file.parent
+        resources.mkdir(parents=True, exist_ok=True)
         if code_blocks:
             for values in code_blocks.values():
                 with open(f"{resources}/{values['path']}", 'w') as f:
                     f.write(values['code'])
 
-        print(f'Writing the HTML file here: {log_file.parent.absolute()}/{log_file.stem}.html')
-        with open(f'{log_file.parent.absolute()}/{log_file.stem}.html', 'w') as f:
+        html_file = Path(f'{resources.absolute()}/{log_file.stem}.html')
+        with html_file.open('w') as f:
             f.write(html)
         # endregion Send Files To Log Directory
 
@@ -1120,7 +1124,7 @@ class HtmlLogGenerator:
             </div>
             <div id="filters-wrapper">
                 <div id="filters" class="decompressed">
-                    <div id="log-tags">
+                    <div id="log-tags" style="grid-template-columns: repeat({min(len(self._log_tags), 4)}, auto">
                         {filters}
                     </div>
                     <div class="divider"></div>
@@ -1270,9 +1274,11 @@ class HtmlLogGenerator:
             </div>
             """
         ]
+        used_log_tags = set()
 
         for e, log_entry in enumerate(log_entries):
             log_tag = self._log_tags[log_entry[self._columns.tag_id]]
+            used_log_tags.add(log_entry[self._columns.tag_id])
             log_entry_id = f'log-entry-{e}'
             msg_block_id = f'msg-block-{e}'
             info_block_id = f'info-block-{e}'
@@ -1339,4 +1345,4 @@ class HtmlLogGenerator:
             </div>
             """)
             e += 1
-        return ''.join(logs)
+        return used_log_tags, ''.join(logs)
